@@ -4,4 +4,5 @@ from enum import Enum
 class SSODetectionStrategy(Enum):
     MANUAL = 1,
     ELEMENT_SEARCH = 2,
-    IMAGE_ANALYSIS = 3
\ No newline at end of file
+    IMAGE_ANALYSIS = 3,
+    NONE = 4
