@@ -1,5 +1,6 @@
 from multiprocessing import Process
 from os.path import exists
+from time import sleep
 
 from selenium.common.exceptions import TimeoutException, WebDriverException
 
@@ -21,13 +22,14 @@ def thread_process(site, sso_detection_service, local_run_results, google, faceb
         logger.info("Analyzing site " + site)
         chromedriver = DriverManager.generate_driver()
         chromedriver.get(site)
+        sleep(10)
         ids = sso_detection_service.automatic_sso_detection(
             chromedriver, LoginPathInformation(PageInformation(-1, site), -1, site, [], True, False, None),
             run_google=google, run_facebook=facebook, run_apple=apple)
         ids_only = []
         for id in ids:
             ids_only.append(id[0])
-        file_service.save_to_file(local_run_results, site, ids_only)
+        file_service.save_to_file(local_run_results, site, ids)
         chromedriver.close()
         chromedriver.quit()
     except (RenewalRequestNeededException, TimeoutException, WebDriverException,
