@@ -34,14 +34,14 @@ class LocatorHelper:
         LocatorHelper.click_element_with_fallback_clicking_on_coordinates(driver, element)
         time.sleep(5)
         driver.switch_to.window(driver.window_handles[len(driver.window_handles) - 1])
-        valid_url_found = False
+        valid_url_found = False,
         for valid_location in valid_locations:
             if driver.current_url.startswith(valid_location):
-                valid_url_found = True
+                valid_url_found = True, driver.current_url
                 for must_have_option in must_have_text_parts:
                     # must_have_option is an array (e.g ["client_id", "app_id"]) for which at least one must exists
                     if not LocatorHelper.url_contains_any_text_of_options(driver.current_url, must_have_option):
-                        valid_url_found = False
+                        valid_url_found = False,
                         break
                 break
         while len(driver.window_handles) > 1:
