@@ -101,10 +101,10 @@ class SocialLoginLocator:
             except WebDriverException as err:
                 logger.error(err.msg)
                 raise ManualAnalysisNeededException(err.msg)
-            if element_found:
+            if element_found[0]:
                 logger.info("Found login redirect from low validity element! Reloading the page...")
                 DriverManager.prepare_webpage_with_steps_to_reproduce(driver, item)
-                return True, element_coordinates, element_size, SSODetectionStrategy.ELEMENT_SEARCH
+                return True, element_coordinates, element_size, SSODetectionStrategy.ELEMENT_SEARCH, element_found[1]
             if driver.current_url != base_url_checks:
                 logger.info("Click changed base page. Must reload site..")
                 elements = self.reload_elements(driver, item, elements, high_validity)
