@@ -98,7 +98,7 @@ class SSODetectionService:
                 ids.append((self.provider_ids['apple'], apple_locate_result[1], apple_locate_result[2],
                             latest_login_info.loginPath, apple_locate_result[3], apple_locate_result[4]))
         if len(ids) == 0:
-            ids.append((self.provider_ids['none'], None, None, None, SSODetectionStrategy.ELEMENT_SEARCH))
+            ids.append((self.provider_ids['none'], None, None, None, SSODetectionStrategy.ELEMENT_SEARCH, None))
         return ids
 
     def manual_sso_detection(self) -> [tuple]:
