@@ -49,7 +49,7 @@ def check_local_run_config(args):
     if not path.isfile(args.local_run_sites):
         logger.error(args.local_run_sites + " does not exist!")
         return False
-    if path.isfile(args.local_run_results):
+    if path.isfile(args.local_run_results) and not args.retry:
         logger.error("Results file at " + args.local_run_results + " already exists!")
         return False
     return True
