@@ -21,6 +21,8 @@ class RestClient:
     path_get_tranco_id_for_site = "/rest/tranco/idforsite"
     path_known_sso_providers = "/rest/knownproviders/all"
     path_file_upload_login_path_detection = "/rest/files/loginlocationdetection"
+    path_post_timing_results = "/rest/timing/new"
+
     # endregion
     # region Login Page Locations Paths
     path_next_page_of_tranco_domains_with_no_login_path_detection = "/rest/loginlocation/getNextNotAnalysedPage"
@@ -123,6 +125,23 @@ class RestClient:
         response = self.send_get_request(self.path_known_sso_providers)
         return RestHelper.convert_json_to_list_of_sso_provider(response)
 
+    def post_timing_results(self, analysis_run_id, base_page, time_login_site_detection, time_keyword_based_analysis,
+                            time_image_analysis, time_combined_analyses, time_complete_process,
+                            time_complete_process_with_upload, sites_analysed):
+        post_data = {'analysisrunid': analysis_run_id, 'basepage': base_page,
+                     "timeloginsitedetection": time_login_site_detection,
+                     "timekeywordbasedanalysis": time_keyword_based_analysis, "timeimageanalysis": time_image_analysis,
+                     "timecombinedanalyses": time_combined_analyses, "timecompleteprocess": time_complete_process,
+                     "timecompleteprocesswithupload": time_complete_process_with_upload,
+                     "sitesanalysed": sites_analysed}
+        response = self.send_post_request(self.path_post_timing_results, post_data)
+        if response.status_code == 200:
+            logger.info("Successfully saved timing results")
+            return True
+        else:
+            logger.error("Something went wrong (code " + str(response.status_code) + ")")
+            return False
+
     # endregion
 
     # region Login Page Locations
@@ -272,6 +291,9 @@ class RestClient:
     def get_search_engine_mode_for_run(self, run_id):
         return self.get_additional_analysis_run_information(run_id, "SEARCH_ENGINE")
 
+    def get_search_algorithms_for_run(self, run_id):
+        return self.get_additional_analysis_run_information(run_id, "SEARCH_ALGORITHM")
+
     # endregion
 
     # region Privacy Detection
@@ -442,6 +464,8 @@ class RestClient:
                                          {'analysisrunid': analysis_run_id, 'key': key})
         if response.status_code == 200 and response.text is not None and len(response.text) != 0:
             return response.text
+        elif response.status_code == 404:
+            return None
         logger.error("Something went wrong (code " + str(response.status_code) + ")")
         return None
 
